@@ -490,6 +490,7 @@ BCH_DEBUG_PARAMS_ALL()
 	x(blocked_journal_low_on_pin)		\
 	x(blocked_journal_max_in_flight)	\
 	x(blocked_journal_max_open)		\
+	x(blocked_journal_write_buffer_flush)	\
 	x(blocked_key_cache_flush)		\
 	x(blocked_allocate)			\
 	x(blocked_allocate_open_bucket)		\
@@ -753,6 +754,7 @@ struct bch_fs {
 	struct kobject		internal;
 	struct kobject		opts_dir;
 	struct kobject		time_stats;
+	struct kobject		time_stats_json;
 	unsigned long		flags;
 
 	int			minor;
@@ -791,7 +793,8 @@ struct bch_fs {
 	struct bch_devs_mask	devs_rotational;
 
 	struct bch_opts		opts;
-	atomic_t		opt_change_cookie;
+	struct mutex		opt_change_lock;
+	u32			opt_change_cookie;
 	struct bch_opts_mask	mount_opts;
 
 	struct bch_sb_cpu	sb;
@@ -903,6 +906,7 @@ struct bch_fs {
 
 static inline int __bch2_err_throw(struct bch_fs *c, int err)
 {
+	BUG_ON(err >= 0);
 	this_cpu_inc(c->counters.now[BCH_COUNTER_error_throw]);
 	trace_error_throw(c, bch2_err_str(err));
 	return err;
