@@ -60,8 +60,6 @@ int __bch2_err_class(int bch_err)
 	while (std_err >= BCH_ERR_START && bch2_errcode_parents[std_err - BCH_ERR_START])
 		std_err = bch2_errcode_parents[std_err - BCH_ERR_START];
 
-	trace_error_downcast(bch_err, std_err, _RET_IP_);
-
 	return -std_err;
 }
 
@@ -71,3 +69,31 @@ const char *bch2_blk_status_to_str(blk_status_t status)
 		return "device removed";
 	return blk_status_to_str(status);
 }
+
+enum bch_errcode blk_status_to_bch_err(blk_status_t err)
+{
+	if (!err)
+		return 0;
+
+	switch (err) {
+#undef BLK_STS
+#define BLK_STS(n) case BLK_STS_##n:	return BCH_ERR_BLK_STS_##n;
+		BLK_ERRS()
+#undef BLK_STS
+		default:		return BCH_ERR_BLK_STS_UNKNOWN;
+	}
+}
+
+enum bch_errcode zstd_err_to_bch_err(ZSTD_ErrorCode err)
+{
+	if (!err)
+		return 0;
+
+	switch (err) {
+#undef ZSTD_error
+#define ZSTD_error(n) case ZSTD_error_##n:	return BCH_ERR_ZSTD_error_##n;
+		ZSTD_ERRS()
+#undef ZSTD_error
+		default:		return BCH_ERR_ZSTD_error_unknown;
+	}
+}
