@@ -12,8 +12,9 @@
 #include "btree/node_scan.h"
 
 #include "data/copygc.h"
-#include "data/ec.h"
-#include "data/reconcile.h"
+#include "data/ec/init.h"
+#include "data/reconcile/check.h"
+#include "data/reconcile/work.h"
 
 #include "fs/check.h"
 #include "fs/inode.h"
@@ -149,6 +150,7 @@ static void bch2_sb_recovery_pass_complete(struct bch_fs *c,
 					   enum bch_recovery_pass pass,
 					   s64 start_time)
 {
+	guard(memalloc_flags)(PF_MEMALLOC_NOFS);
 	guard(mutex)(&c->sb_lock);
 	struct bch_sb_field_ext *ext = bch2_sb_field_get(c->disk_sb.sb, ext);
 	__clear_bit_le64(bch2_recovery_pass_to_stable(pass),
@@ -171,6 +173,7 @@ static void bch2_sb_recovery_pass_complete(struct bch_fs *c,
 void bch2_recovery_pass_set_no_ratelimit(struct bch_fs *c,
 					 enum bch_recovery_pass pass)
 {
+	guard(memalloc_flags)(PF_MEMALLOC_NOFS);
 	guard(mutex)(&c->sb_lock);
 
 	struct recovery_pass_entry *e = bch2_sb_recovery_pass_entry(c, pass);
@@ -215,6 +218,7 @@ static bool bch2_recovery_pass_want_ratelimit_locked(struct bch_fs *c, enum bch_
 bool bch2_recovery_pass_want_ratelimit(struct bch_fs *c, enum bch_recovery_pass pass,
 				       unsigned runtime_fraction)
 {
+	guard(memalloc_flags)(PF_MEMALLOC_NOFS);
 	guard(mutex)(&c->sb_lock);
 	return bch2_recovery_pass_want_ratelimit_locked(c, pass, runtime_fraction);
 }
@@ -445,6 +449,7 @@ int bch2_run_explicit_recovery_pass(struct bch_fs *c,
 	    !recovery_pass_needs_set(c, pass, &flags))
 		return 0;
 
+	guard(memalloc_flags)(PF_MEMALLOC_NOFS);
 	guard(mutex)(&c->sb_lock);
 	bool write_sb = false;
 	int ret = __bch2_run_explicit_recovery_pass(c, out, pass, flags, &write_sb);
@@ -466,6 +471,7 @@ int bch2_require_recovery_pass(struct bch_fs *c,
 	    c->recovery.passes_complete & BIT_ULL(pass))
 		return 0;
 
+	guard(memalloc_flags)(PF_MEMALLOC_NOFS);
 	guard(mutex)(&c->sb_lock);
 
 	if (bch2_recovery_pass_want_ratelimit_locked(c, pass, 100))
