@@ -18,6 +18,7 @@
 #include <linux/percpu.h>
 #include <linux/preempt.h>
 #include <linux/random.h>
+#include <linux/seq_buf.h>
 #include <linux/seq_file.h>
 #include <linux/string.h>
 #include <linux/types.h>
@@ -463,6 +464,17 @@ void bch2_time_stats_to_text(struct printbuf *out, struct bch2_time_stats *stats
 	}
 }
 
+void bch2_time_stats_json_to_text(struct printbuf *out, struct bch2_time_stats *stats,
+				  const char *epoch_name, unsigned int flags)
+{
+	char buf[1024];
+	struct seq_buf seq;
+
+	seq_buf_init(&seq, buf, sizeof(buf));
+	bch2_time_stats_to_json(&seq, stats, epoch_name, flags);
+	prt_bytes(out, buf, seq_buf_used(&seq));
+}
+
 /* ratelimit: */
 
 /**
@@ -606,49 +618,32 @@ void bch2_bio_map(struct bio *bio, void *base, size_t size)
 
 int bch2_bio_alloc_pages(struct bio *bio, unsigned bs, size_t size, gfp_t gfp_mask)
 {
+	BUG_ON(!is_power_of_2(bs));
 	BUG_ON(size & (bs - 1));
-	unsigned bs_pages = DIV_ROUND_UP(bs, PAGE_SIZE);
 
-	/*
-	 * XXX: we could do this by allocating higher order pages, but
-	 *
-	 * - the page allocator gets slower at a certain order (5?) - we'd have
-	 *   to check for this
-	 *
-	 * - bch2_bio_free_pages_pool() probably does not handle compound pages
-	 *   yet
-	 */
-	DARRAY_PREALLOCATED(struct page *, 16) pages;
-	darray_init(&pages);
-	darray_make_room_gfp(&pages, bs_pages, gfp_mask|__GFP_NOFAIL);
+	unsigned max_alloc = max(bs, PAGE_SIZE << PAGE_ALLOC_COSTLY_ORDER);
 
-	int ret = 0;
-	while (size) {
-		while (pages.nr < bs_pages) {
-			struct page *page = alloc_pages(gfp_mask, 0);
-			if (!page) {
-				ret = -ENOMEM;
-				goto out;
-			}
+	while (bio->bi_iter.bi_size < size) {
+		unsigned b = min(size - bio->bi_iter.bi_size, max_alloc);
 
-			BUG_ON(darray_push(&pages, page));
-		}
+		BUG_ON(b & (bs - 1));
 
-		while (pages.nr) {
-			BUG_ON(!size);
+#ifdef __KERNEL__
+		/*
+		 * we don't know the device dma alignment, so in kernel make
+		 * sure allocations are page aligned
+		 */
+		void *p = (void *) __get_free_pages(gfp_mask, get_order(b));
+#else
+		void *p = kmalloc(b, gfp_mask);
+#endif
+		if (!p)
+			return -ENOMEM;
 
-			unsigned len = min(PAGE_SIZE, size);
-			size -= len;
-
-			struct page *page = darray_pop(&pages);
-			BUG_ON(!bio_add_page(bio, page, len, 0));
-		}
+		bio_add_virt_nofail(bio, p, b);
 	}
-out:
-	darray_for_each(pages, i)
-		__free_page(*i);
-	darray_exit(&pages);
-	return ret;
+
+	return 0;
 }
 
 u64 bch2_get_random_u64_below(u64 ceil)
@@ -678,9 +673,8 @@ void memcpy_to_bio(struct bio *dst, struct bvec_iter dst_iter, const void *src)
 	struct bvec_iter iter;
 
 	__bio_for_each_segment(bv, dst, iter, dst_iter) {
-		void *dstp = kmap_local_page(bv.bv_page);
-
-		memcpy(dstp + bv.bv_offset, src, bv.bv_len);
+		void *dstp = bvec_kmap_local(&bv);
+		memcpy(dstp, src, bv.bv_len);
 		kunmap_local(dstp);
 
 		src += bv.bv_len;
@@ -693,9 +687,8 @@ void memcpy_from_bio(void *dst, struct bio *src, struct bvec_iter src_iter)
 	struct bvec_iter iter;
 
 	__bio_for_each_segment(bv, src, iter, src_iter) {
-		void *srcp = kmap_local_page(bv.bv_page);
-
-		memcpy(dst, srcp + bv.bv_offset, bv.bv_len);
+		void *srcp = bvec_kmap_local(&bv);
+		memcpy(dst, srcp, bv.bv_len);
 		kunmap_local(srcp);
 
 		dst += bv.bv_len;
@@ -1008,13 +1001,6 @@ u64 *bch2_acc_percpu_u64s(u64 __percpu *p, unsigned nr)
 	return ret;
 }
 
-void bch2_darray_str_exit(darray_const_str *d)
-{
-	darray_for_each(*d, i)
-		kfree(*i);
-	darray_exit(d);
-}
-
 int bch2_split_devs(const char *_dev_name, darray_const_str *ret)
 {
 	darray_init(ret);
@@ -1038,6 +1024,19 @@ int bch2_split_devs(const char *_dev_name, darray_const_str *ret)
 
 	return 0;
 err:
-	bch2_darray_str_exit(ret);
+	darray_exit_free_item(ret, kfree);
 	return -ENOMEM;
 }
+
+#if !defined(__KERNEL__) || LINUX_VERSION_CODE >= KERNEL_VERSION(6,19,0)
+void *mempool_kvmalloc(gfp_t gfp_mask, void *pool_data)
+{
+	size_t size = (size_t)pool_data;
+	return kvmalloc(size, gfp_mask);
+}
+
+void mempool_kvfree(void *element, void *pool_data)
+{
+	kvfree(element);
+}
+#endif
