@@ -24,14 +24,6 @@
 
 /* O_DIRECT reads */
 
-struct dio_read {
-	struct closure			cl;
-	struct kiocb			*req;
-	long				ret;
-	bool				should_dirty;
-	struct bch_read_bio		rbio;
-};
-
 static void bio_check_or_release(struct bio *bio, bool check_dirty)
 {
 	if (check_dirty) {
@@ -52,10 +44,11 @@ static CLOSURE_CALLBACK(bch2_dio_read_complete)
 
 static void bch2_direct_IO_read_endio(struct bio *bio)
 {
+	struct bch_read_bio *rbio = to_rbio(bio);
 	struct dio_read *dio = bio->bi_private;
 
-	if (bio->bi_status)
-		dio->ret = blk_status_to_errno(bio->bi_status);
+	if (rbio->ret)
+		dio->ret = bch2_err_class(rbio->ret);
 
 	closure_put(&dio->cl);
 }
@@ -105,7 +98,7 @@ static int bch2_direct_IO_read(struct kiocb *req, struct iov_iter *iter)
 			       bio_iov_vecs_to_alloc(iter, BIO_MAX_VECS),
 			       REQ_OP_READ,
 			       GFP_KERNEL,
-			       &c->dio_read_bioset);
+			       &c->vfs.dio_read_bioset);
 
 	dio = container_of(bio, struct dio_read, rbio.bio);
 	closure_init(&dio->cl, NULL);
@@ -153,7 +146,7 @@ start:
 		ret = bch2_bio_iov_iter_get_pages(bio, iter, 0);
 		if (ret < 0) {
 			/* XXX: fault inject this path */
-			bio->bi_status = BLK_STS_RESOURCE;
+			to_rbio(bio)->ret = ret;
 			bio_endio(bio);
 			break;
 		}
@@ -232,26 +225,6 @@ out:
 
 /* O_DIRECT writes */
 
-struct dio_write {
-	struct kiocb			*req;
-	struct address_space		*mapping;
-	struct bch_inode_info		*inode;
-	struct mm_struct		*mm;
-	const struct iovec		*iov;
-	unsigned			loop:1,
-					extending:1,
-					sync:1,
-					flush:1;
-	struct quota_res		quota_res;
-	u64				written;
-
-	struct iov_iter			iter;
-	struct iovec			inline_vecs[2];
-
-	/* must be last: */
-	struct bch_write_op		op;
-};
-
 static bool bch2_check_range_allocated(struct bch_fs *c, subvol_inum inum,
 				       u64 offset, u64 size,
 				       unsigned nr_replicas, bool compressed)
@@ -278,7 +251,7 @@ retry:
 
 		if (k.k->p.snapshot != snapshot ||
 		    nr_replicas > bch2_bkey_replicas(c, k) ||
-		    (!compressed && bch2_bkey_sectors_compressed(k)))
+		    (!compressed && bch2_bkey_sectors_compressed(c, k)))
 			return false;
 	}
 err:
@@ -443,6 +416,13 @@ static __always_inline void bch2_dio_write_end(struct dio_write *dio)
 		set_bit(EI_INODE_ERROR, &inode->ei_flags);
 }
 
+static void bch2_dio_write_sync_done(struct bch_write_op *op)
+{
+	struct dio_write *dio = container_of(op, struct dio_write, op);
+
+	dio->sync_done = true;
+}
+
 static __always_inline long bch2_dio_write_loop(struct dio_write *dio)
 {
 	struct bch_fs *c = dio->op.c;
@@ -506,7 +486,7 @@ static __always_inline long bch2_dio_write_loop(struct dio_write *dio)
 
 		bch2_write_op_init(&dio->op, c, opts);
 		dio->op.end_io		= sync
-			? NULL
+			? bch2_dio_write_sync_done
 			: bch2_dio_write_loop_async;
 		dio->op.target		= dio->op.opts.foreground_target;
 		dio->op.write_point	= writepoint_hashed((unsigned long) current);
@@ -544,6 +524,9 @@ static __always_inline long bch2_dio_write_loop(struct dio_write *dio)
 		if (!sync)
 			return -EIOCBQUEUED;
 
+		BUG_ON(!dio->sync_done);
+		dio->sync_done = false;
+
 		bch2_dio_write_end(dio);
 
 		if (likely(!dio->iter.count) || dio->op.error)
@@ -638,7 +621,7 @@ ssize_t bch2_direct_write(struct kiocb *req, struct iov_iter *iter)
 			       bio_iov_vecs_to_alloc(iter, BIO_MAX_VECS),
 			       REQ_OP_WRITE | REQ_SYNC | REQ_IDLE,
 			       GFP_KERNEL,
-			       &c->dio_write_bioset);
+			       &c->vfs.dio_write_bioset);
 	dio = container_of(bio, struct dio_write, op.wbio.bio);
 	dio->req		= req;
 	dio->mapping		= mapping;
@@ -676,25 +659,4 @@ err_put_write_ref:
 	goto out;
 }
 
-void bch2_fs_fs_io_direct_exit(struct bch_fs *c)
-{
-	bioset_exit(&c->dio_write_bioset);
-	bioset_exit(&c->dio_read_bioset);
-}
-
-int bch2_fs_fs_io_direct_init(struct bch_fs *c)
-{
-	if (bioset_init(&c->dio_read_bioset,
-			4, offsetof(struct dio_read, rbio.bio),
-			BIOSET_NEED_BVECS))
-		return -BCH_ERR_ENOMEM_dio_read_bioset_init;
-
-	if (bioset_init(&c->dio_write_bioset,
-			4, offsetof(struct dio_write, op.wbio.bio),
-			BIOSET_NEED_BVECS))
-		return -BCH_ERR_ENOMEM_dio_write_bioset_init;
-
-	return 0;
-}
-
 #endif /* NO_BCACHEFS_FS */
